#ifndef PW_SCRIPTING_RUNTIME_LUA_HPP
#define PW_SCRIPTING_RUNTIME_LUA_HPP

#include "sol/sol.hpp"
#include <lua.hpp>
#include <map>

namespace pw {

class runtime_lua {
public:
	typedef std::function<void(sol::state&,sol::table&)> register_function_t;
	typedef std::map<std::string,register_function_t> register_func_list_t;

	static runtime_lua& get();

	register_func_list_t register_functions() { return _register_function_list; }

	void add(const std::string& name,register_function_t f);

protected:

	runtime_lua();

	register_func_list_t _register_function_list;

};

template<typename Derived>
struct runtime_lua_register
{
	Derived& derived() { return static_cast<Derived&>(*this); }
	const Derived& derived() const { return static_cast<const Derived&>(*this); }

	runtime_lua_register()
	{
		printf("Test\n");
		runtime_lua::get().add(derived().module_name(),&Derived::register_function);
	}

	static runtime_lua_register<Derived> _instance;
};


extern "C" {
	typedef void (* CModuleFunction) (void);
}

struct ModuleFunctionProxy
{
	ModuleFunctionProxy(CModuleFunction function) { (function)(); }
};

}

#define PW_RUNTIME_LUA_USE(name) \
	extern "C" void pw_runtime_lua_##name(void) { printf("Meh!\n"); };


#define PW_RUNTIME_LUA_REGISTER(name) \
	extern "C" void pw_runtime_lua_##name(void); \
	static pw::ModuleFunctionProxy proxy_##name(pw_runtime_lua_##name);

#endif
