#include "pw/scripting/script.hpp"

#include "runtime_lua.hpp"

#include "pw/core/debug.hpp"

namespace pw {

struct script::state {

	sol::state _state;
	sol::table _namespace;

	state();

	void load_modules();
	void load(const std::string& name);

	int run(const std::string &s);

};

int script::state::run(const std::string &s)
{
	return _state.script(s.c_str()).get<int>();
}

void script::state::load(const std::string& name)
{
	runtime_lua::get().register_functions()[name](_state,_namespace);
}

script::state::state()
{
	// create global pw namespace
	_namespace = _state.create_named_table("pw");


	// add the script as a user type
	_namespace.new_usertype<state>("script",
								   "initialize",&state::load_modules,
								   "load",&state::load
								   );

	// add this instance as "script"
	_namespace.set("script",this);

	debug::d() << "available " << runtime_lua::get().register_functions().size();

}

void script::state::load_modules() {

	// open all libraries
	_state.open_libraries();
}

//
// script
//

script::script()
	: _state(make_unique<script::state>())
{
}

script::~script()
{
}

int script::eval(const std::string &s)
{
	return _state->run(s);
}


}




